# -*- coding: utf-8 -*-
# Apache Software License 2.0
#
# Copyright (c) 2018, Christophe Duong
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from logging import info
from threading import Thread
from subprocess import Popen, PIPE, STDOUT


def find(collection, item, field='name'):
    """
    Finds an element in a collection which has a field equal
    to particular item value

    Parameters
    ----------
    collection : Set
        Collection of objects
    item
        value of the item that we are looking for
    field : string
        Name of the field from the object to inspect

    Returns
    -------
    object
        Corresponding element that has a field matching item in
        the collection
    """
    for s in collection:
        if s[field] == item:
            return s
    return None


def copy_replace(src, dst, pattern='', replace_value=''):
    """
    Copies a file from src to dst replacing pattern by replace_value

    Parameters
    ----------
    src : string
        Path to the source filename to copy from
    dst : string
        Path to the output filename to copy to
    pattern : string
        Pattern to replace inside the src file
    replace_value
        Value to replace by in the dst file

    """
    f1 = open(src, 'r')
    f2 = open(dst, 'w')
    for line in f1:
        f2.write(line.replace(pattern, replace_value))
    f1.close()
    f2.close()


def log_info(pipe):
    """ Default logging function """
    for line in iter(pipe.readline, b''):
        info(line)
    return True


class BackgroundThreadRunner(object):
    """
    Worker Thread to run logging output in the background

    ...

    Attributes
    ----------
    process :
        Process object of the command running in the background
    log_function : function(stream -> bool)
        callback function to log the output of the command
    no_redirect : bool
        whether the subprocess STDOUT and STDERR should be redirected to logs
    worker : Thread
        Thread object
    """
    def __init__(self, command, log_function, no_redirect=False):
        self.no_redirect = no_redirect
        if no_redirect:
            self.process = Popen(command)
        else:
            self.process = Popen(command, stdout=PIPE, stderr=STDOUT)
        self.log_function = log_function
        self.worker = Thread(name='worker', target=self.run)
        self.worker.start()

    def run(self):
        if not self.no_redirect:
            self.log_function(self.process.stdout)


def subprocess_run(command, log_function=log_info,
                   no_redirect=False, wait=True):
    """
    Run command in a subprocess while redirecting output to log_function.

    The subprocess either runs synchroneoulsy or in the background depending on
    the wait parameter.

    Parameters
    ----------
    command : List
        Command to run in the subprocess
    log_function : function
        Callback function to log the output of the subprocess
    no_redirect : bool
        whether the subprocess STDOUT and STDERR should be redirected to logs
    wait : bool
        Whether the subprocess should be run synchroneously or in
        the background
    Returns
    -------
    int
        return code of the subprocess
    BackgroundThreadRunner
        the thread running in the background
    """
    if wait:
        if no_redirect:
            process = Popen(command)
        else:
            process = Popen(command, stdout=PIPE, stderr=STDOUT)
            with process.stdout:
                log_function(process.stdout)
        return process.wait()
    else:
        return BackgroundThreadRunner(command, log_function, no_redirect)



# -*- coding: utf-8 -*-
# Apache Software License 2.0
#
# Copyright (c) 2018, Christophe Duong
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Implementations of commands for Jupyter
"""
from logging import info, debug
from os import chdir, getcwd, makedirs, listdir
from os.path import basename, dirname, abspath, isfile, join
from shutil import copy
from tempfile import TemporaryDirectory
from time import sleep
import webbrowser

from aiscalator.core.utils import copy_replace, subprocess_run
from aiscalator.core.config import find_global_config_file, AiscalatorConfig
from aiscalator.core.log_regex_analyzer import LogRegexAnalyzer


def docker_build(step: AiscalatorConfig):
    """
    Builds the docker image following the parameters specified in the
    focused step's configuration file

    Parameters
    ----------
    step : AiscalatorConfig
        Configuration object for this step

    Returns
    -------
    string
        the docker artifact name of the image built
    """
    # Retrieve configuration parameters
    # TODO check validity of dockerfilename (choice are jupyter-spark)
    dockerfilename = step.step_field('dockerfile')
    if dockerfilename is None:
        dockerfilename = "jupyter-spark"
    dockerfiledir = dirname(find_global_config_file(
        "config/docker/" + dockerfilename + "/Dockerfile"
    ))
    docker_image_name = step.step_field('dockerImageName')
    requirements = step.file_path('requirementsPath')
    cwd = getcwd()
    try:
        # Prepare a temp folder to build docker image
        with TemporaryDirectory(prefix="aiscalator_") as tmp:
            # copy the dockerfile
            if requirements is not None:
                copy_replace(dockerfiledir + "/Dockerfile",
                             tmp + '/Dockerfile',
                             '#requirements.txt#', """
COPY requirements.txt requirements.txt
RUN pip install -r requirements.txt
RUN rm requirements.txt"""
                             )
                copy(requirements, tmp + '/requirements.txt')
            else:
                copy(dockerfiledir + "/Dockerfile", tmp + '/Dockerfile')
            for f in listdir(dockerfiledir):
                if f != "Dockerfile":
                    copy(join(dockerfiledir, f), join(tmp, f))
            chdir(tmp)
            debug("Running...: docker build --rm -t " +
                  docker_image_name + " .")
            logger = LogRegexAnalyzer(b'Successfully built ([a-zA-Z0-9]+)\n')
            subprocess_run([
                "docker", "build", "--rm", "-t", docker_image_name, "."
            ], log_function=logger.grep_logs)
            result = logger.artifact
    finally:
        chdir(cwd)
    return result


def prepare_docker_run_notebook(step: AiscalatorConfig, program):
    """
    Assembles the list of commands to execute a docker run call

    When calling "docker run ...", this function also adds a set of
    additional parameters to mount the proper volumes and expose the
    correct environment for the call in the docker image mapped to the
    host directories. This is done so only some specific data and code
    folders are accessible within the docker image.

    Parameters
    ----------
    step : AiscalatorConfig
        Configuration object for the step
    program : List
        the rest of the commands to execute as part of
        the docker run call

    Returns
    -------
    List
        The full Array of Strings representing the commands to execute
        in the docker run call
    """
    commands = [
        "docker", "run", "--name", step.container_name(), "--rm",
        "-p", "10000:8888",
        "-p", "4040:4040"
     ]
    for env in step.user_env_file():
        if isfile(env):
            commands += ["--env-file", env]
    # TODO improve port publishing
    filename = (step.file_path('path'))
    commands += [
        "--mount", "type=bind,source=" + dirname(filename) +
        ",target=/home/jovyan/work/notebook/",
        "--mount", "type=bind,source=" + abspath(step.config_path) +
                   ",target=/home/jovyan/work/" + basename(step.config_path),
    ]
    requirements = step.file_path('requirementsPath')
    if requirements is not None:
        commands += [
            "--mount", "type=bind,source=" + requirements +
                       ",target=/home/jovyan/work/requirements.txt",
        ]
    for v in step.step_field("modulesPath"):
        # TODO support readonly flag in config & bind vs tmpfs types
        for i in v:
            # TODO check if v[i] is relative path?
            commands += [
                "--mount", "type=bind,source=" + abspath(step.rootDir + i) +
                ",target=/home/jovyan/work/modules/" + v[i]
            ]
    for v in step.step_field("inputPath"):
        for i in v:
            # TODO check if v[i] is relative path?
            commands += [
                "--mount", "type=bind,source=" + abspath(step.rootDir + i) +
                ",target=/home/jovyan/work/data/input/" + v[i] +
                ",readonly"
            ]
    for v in step.step_field("outputPath"):
        for i in v:
            # TODO check if v[i] is relative path?
            makedirs(abspath(step.rootDir + i), exist_ok=True)
            commands += [
                "--mount", "type=bind,source=" + abspath(step.rootDir + i) +
                ",target=/home/jovyan/work/data/output/" + v[i]
            ]
    # TODO if filename does not exist, generate one
    commands += program
    debug("Running...: " + " ".join(commands))
    return commands


def docker_run_papermill(step: AiscalatorConfig, prepare_only=False):
    """
    Executes the step in browserless mode using papermill

    Parameters
    ----------
    step : AiscalatorConfig
        Configuration object for the step
    prepare_only : bool
        Indicates if papermill should replace the parameters of the
        notebook only or it should execute all the cells too

    Returns
    -------
    string
        the path to the output notebook resulting from the execution
        of this step
    """
    docker_image = docker_build(step)
    notebook = ("/home/jovyan/work/notebook/" +
                basename(step.file_path('path')))
    notebook_output = step.notebook_output_path(notebook)
    parameters = step.extract_parameters()
    makedirs(step.file_path('executionDirPath'), exist_ok=True)
    commands = prepare_docker_run_notebook(step, [
            "--mount", "type=bind,source=" +
                       step.file_path('executionDirPath') +
            ",target=/home/jovyan/work/notebook_run/",
            docker_image, "start.sh",
            # TODO: check step type, if jupyter then papermill
            "papermill", notebook, notebook_output
    ])
    if prepare_only:
        commands.append("--prepare-only")
    commands += parameters
    logger = LogRegexAnalyzer()
    # TODO output log in its own execution log file
    subprocess_run(commands, log_function=logger.grep_logs)
    # TODO handle notebook_output execution history and latest successful run
    return notebook_output


def docker_run_lab(step: AiscalatorConfig):
    """
    Starts a Jupyter Lab environment configured to edit the focused step

    Parameters
    ----------
    step : AiscalatorConfig
        Configuration object for the step

    Returns
    -------
    string
        Url of the running jupyter lab
    """
    docker_image = docker_build(step)
    # TODO: shutdown other jupyter lab still running
    notebook = basename(step.step_field('path'))
    if len(step.extract_parameters()) > 0:
        docker_run_papermill(step, prepare_only=True)
    makedirs(step.file_path('executionDirPath'), exist_ok=True)
    commands = [
        "--mount",
        "type=bind,source=" + step.file_path('executionDirPath') +
        ",target=/home/jovyan/work/notebook_run/",
    ]
    commands = prepare_docker_run_notebook(step, commands + [
        docker_image, "start.sh", 'jupyter', 'lab'
    ])
    logger = LogRegexAnalyzer(b'http://.*:8888/.token=([a-zA-Z0-9]+)\n')
    subprocess_run(commands, log_function=logger.grep_logs, wait=False)
    for i in range(5):
        sleep(2)
        if logger.artifact != "latest":
            break
        info("docker run does not seem to be up yet... retrying ("
             + str(i) + "/5)")
    if logger.artifact != "latest":
        # TODO handle url better (not always localhost?)
        url = ("http://localhost:10000/lab/tree/work/notebook/" +
               notebook + "?token=" +
               logger.artifact)
        info(url + " is up and running.")
        # TODO --no-browser option
        webbrowser.open(url)
        return url
    else:
        return ""



# -*- coding: utf-8 -*-
# Apache Software License 2.0
#
# Copyright (c) 2018, Christophe Duong
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
CLI module for Jupyter related commands.
"""
import click
import logging

from aiscalator import __version__
from aiscalator.core.config import AiscalatorConfig
from aiscalator.jupyter import docker_command


@click.group()
@click.version_option(version=__version__)
def jupyter():
    """Notebook environment to explore and handle data."""
    pass


@jupyter.command()
def setup():
    """Setup the docker image to run notebooks."""
    # TODO: ask questions and generates json + ipynb files
    # TODO: afterward edit the newly created step
    logging.error("Not implemented yet")
    pass


@jupyter.command()
def new():
    """Create a new notebook file with a new aiscalate config."""
    # TODO: ask questions and generates json + ipynb files
    # TODO: afterward edit the newly created step
    logging.error("Not implemented yet")
    pass


@jupyter.command()
@click.argument('conf')
@click.argument('notebook', nargs=-1)
# TODO add parameters override from CLI
def edit(conf, notebook):
    """Edit the notebook from an aiscalate config with JupyterLab."""
    click.echo(docker_command.docker_run_lab(AiscalatorConfig(conf, notebook)))


@jupyter.command()
@click.argument('conf')
@click.argument('notebook', nargs=-1)
# TODO add parameters override from CLI
def run(conf, notebook):
    """Run the notebook from an aiscalate config without GUI."""
    # TODO run multiple notebooks
    # we have to stage notebooks with same dockerfile together,
    # merge their requirements so that groups of notebooks can be
    # run together in the same container sequentially
    click.echo(docker_command.docker_run_papermill(
        AiscalatorConfig(conf, notebook))
    )



# -*- coding: utf-8 -*-
# Apache Software License 2.0
#
# Copyright (c) 2018, Christophe Duong
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
CLI module for Airflow related commands.
"""
import click
import logging

from aiscalator import __version__
from aiscalator.core.config import AiscalatorConfig
from aiscalator.airflow import airflow_command


@click.group()
@click.version_option(version=__version__)
def airflow():
    """Author workflow DAGs and run tasks on schedule."""
    pass


@airflow.command()
def setup():
    """Setup interactively the Airflow home folder and configurations."""
    click.echo(airflow_command.airflow_setup(AiscalatorConfig()))


@airflow.command()
def start():
    """Start docker images to bring airflow services up."""
    click.echo(airflow_command.airflow_up(AiscalatorConfig()))
    click.echo("""
Airflow: http://localhost:8080
Flower: http://localhost:5555
               """)


@airflow.command()
def stop():
    """Stop docker images to bring airflow services down."""
    click.echo(airflow_command.airflow_down(AiscalatorConfig()))


@airflow.command()
@click.option("-s", "--service", default="webserver",
              help='Run subcommand in docker service (default webserver)',
              metavar='<service>')
@click.argument('subcommand', nargs=-1, required=True)
def run(service, subcommand):
    """Run sub-command in a running docker service."""
    if len(subcommand) == 0:
        subcommand = None
    click.echo(airflow_command.airflow_cmd(AiscalatorConfig(),
                                           service=service, cmd=subcommand))

# TODO CLI to  scale celery workers
# docker-compose -f docker-compose-CeleryExecutor.yml scale worker=5


@airflow.command()
def new():
    """Create a new DAG job"""
    # TODO to implement
    logging.error("Not implemented yet")
    pass


@airflow.command()
def edit():
    """Edit DAG job"""
    # TODO to implement
    logging.error("Not implemented yet")
    pass


@airflow.command()
def push():
    """Push a job into the DAGS folder to schedule in Airflow."""
    # TODO to implement
    logging.error("Not implemented yet")
    pass



# -*- coding: utf-8 -*-
# Apache Software License 2.0
#
# Copyright (c) 2018, Christophe Duong
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Implementations of commands for Airflow
"""
import logging
import os
from aiscalator.core.config import AiscalatorConfig
from aiscalator.core.utils import subprocess_run


def docker_compose(conf: AiscalatorConfig):
    """
    Prepare a docker-compose command

    Parameters
    ----------
    conf : AiscalatorConfig
       Configuration object for the application
    Returns
    -------
    List
        The array of commands to start a docker-compose
        with the proper parameters
    """
    dockerfile = conf.find_user_config_file(
        "config/docker-compose-CeleryExecutor.yml"
    )
    commands = ["docker-compose"]
    for env in conf.user_env_file():
        if os.path.isfile(env):
            commands += ["--env-file", env]
    commands += ["-f", dockerfile]
    return commands


def airflow_setup(conf: AiscalatorConfig):
    """
    Setup the airflow configuration files and environment

    Parameters
    ----------
    conf : AiscalatorConfig
        Configuration object for the application

    """
    # docker build --build-arg DOCKER_GID=`getent group docker |
    # cut -d ':' -f 3` --rm -t aiscalator/airflow .
    # TODO : to implement
    logging.error("Not implemented yet")
    pass


def airflow_up(conf: AiscalatorConfig):
    """
    Starts an airflow environment

    Parameters
    ----------
    conf : AiscalatorConfig
        Configuration object for the application

    """
    commands = docker_compose(conf) + ["up", "-d"]
    subprocess_run(commands, no_redirect=True)


def airflow_down(conf: AiscalatorConfig):
    """
    Stop an airflow environment

    Parameters
    ----------
    conf : AiscalatorConfig
        Configuration object for the application

    """
    commands = docker_compose(conf) + ["down"]
    subprocess_run(commands, no_redirect=True)


def airflow_cmd(conf: AiscalatorConfig, service="webserver", cmd=None):
    """
    Execute an airflow subcommand

    Parameters
    ----------
    conf : AiscalatorConfig
        Configuration object for the application
    service : string
        service name of the container where to run the command
    cmd : list
        subcommands to run
    """
    commands = docker_compose(conf) + [
        "run", "--rm", service,
    ]
    if cmd is not None:
        commands += cmd
    else:
        commands += ["airflow"]
    subprocess_run(commands, no_redirect=True)



# -*- coding: utf-8 -*-
# Apache Software License 2.0
#
# Copyright (c) 2018, Christophe Duong
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from datetime import datetime
import json
import logging
from logging import config
import os
from pytz import timezone

from platform import uname
from yaml import safe_load
from aiscalator import __version__


from aiscalator.core.utils import find


def find_global_config_file(filename, setup=(lambda x: x), env_key=""):
    """Looks for configuration files and run the setup function with it:

    This function will try to find the configuration filename at multiple
    locations:

        - From environment variable specified as env_key
        - From the development source tree: resources/
        - From user's home directory: ~/.aiscalator/
        - From virtual environment installation folder of data_files as
          specified in the setup.py

    Parameters
    ----------
    filename : string
        filename of the configuration file to find
    setup : function(x -> x)
        callback function to run with the valid configuration file found
    env_key : string
        environment variable name pointing to configuration file to load

    Returns
    -------
    string
        The path of the configuration that was found
    """
    for loc in [
        # from the environment variables
        os.environ.get(env_key, ''),
        # from the development folder
        "resources",
        # from user home
        os.path.expanduser("~") + '/.aiscalator/',
        # from virtual environment install
        os.getenv('VIRTUAL_ENV', '') + '/etc/aiscalator/'
    ]:
        try:
            path = os.path.abspath(os.path.join(loc, filename))
            with open(path, 'rt') as f:
                setup(f)
            return path
        except IOError:
            pass
    setup(None)
    return None


def setup_logging():
    """ Setup the logging configuration of the application """
    log_level = os.getenv('AISCALATOR_LOG_LEVEL', None)
    path = find_global_config_file(
        "config/logging.yaml", load_logging_conf, 'AISCALATOR_LOG_FILE'
    )
    if path is None:
        logging.basicConfig(level=logging.INFO)
    if log_level is not None:
        logging.root.setLevel(log_level)
    msg = ("Starting " + os.path.basename(__name__) +
           " version " + __version__ + " on " +
           "_".join(uname()).replace(" ", "_"))
    logging.debug(msg)


def load_logging_conf(file):
    """Reads and loads the logging configuration file"""
    if file is not None:
        os.makedirs('/tmp/aiscalator/log', exist_ok=True)
        conf = safe_load(file.read())
        config.dictConfig(conf)


def generate_global_config():
    """Generate a standard configuration file for the application in the
    user's home folder ~/.aiscalator/config/config.json
    """
    # TODO when gconf file is not found, generate a new one from a template
    return ""


class AiscalatorConfig(object):
    """
    A configuration object for the Aiscalator application.

    This object stores:
        - global configuration for the whole application
        - configuration for a particular context specified in a step
          configuration file.
        - In this case, we might even focus on a particular step.

    ...

    Attributes
    ----------
    config_path : string
        path to the step configuration file
    rootDir : string
        path to the directory containing the configuration file
    conf : string
        step configuration after loading the file
    step : string
        step name the application is currently focusing on
    """
    def __init__(self, step_config_path=None, notebook=None):
        """
        Parameters
            ----------
            step_config_path : string
                path to the step configuration file
            notebook : string
                name of the step from the configuration file to focus on
        """
        self.config_path = step_config_path
        if step_config_path is None:
            self.rootDir = None
        else:
            self.rootDir = os.path.dirname(step_config_path)
            if not self.rootDir.endswith("/"):
                self.rootDir += "/"
        setup_logging()
        # TODO find user config, if not found then
        # find global config and create one for the user
        find_global_config_file(
            "config/config.json", self.setup_app_config, "AISCALATOR_CONFIG"
        )
        self.conf = self.setup_step_config()
        self.step = self.focus_step(notebook)

    def setup_app_config(self, file):
        """Setup application configuration"""
        if file is None:
            file = generate_global_config()
        print(file)
        # TODO: different formats, merge multiple files etc

    def setup_step_config(self):
        """Setup step configuration"""
        if self.config_path is None:
            return None
        try:
            j = json.loads(self.config_path)
        except json.decoder.JSONDecodeError:
            try:
                with open(self.config_path, "r") as f:
                    j = json.load(f)
            except Exception as err2:
                logging.error("Invalid configuration file")
                raise err2
        logging.debug("Configuration file = " + json.dumps(j, indent=4))
        return j

    def focus_step(self, notebook):
        """Find the notebook in the step configuration"""
        result = None
        if self.config_path is None:
            return None
        if notebook is None or len(notebook) == 0:
            result = self.conf['step'][0]
        else:
            step = find(self.conf['step'], notebook[0])
            if step is not None:
                result = step
            elif notebook == "_":
                result = self.conf['step'][0]
        if result is None:
            raise ValueError("Unknown step " + notebook[0])
        return result

    def find_user_config_file(self, filename):
        """
        Looks for configuration files in the user configuration folder

        Parameters
        ----------
        filename : string
            file to search for

        Returns
        -------
        string
            path to the filename in the user configuration folder
        """
        # TODO check user_config_folder override in environment
        # TODO check if user_config_folder has been redefined on command line
        return os.path.expanduser("~") + '/.aiscalator/' + filename

    def user_env_file(self):
        """
        Find a list of env files to pass to docker containers

        Returns
        -------
        List
            env files
        """
        # TODO look if env file has been defined in the focused step
        # TODO look in user config if env file has been redefined
        return [self.find_user_config_file("config/.env")]

    def notebook_output_path(self, notebook):
        """Generates the name of the output notebook"""
        return ("/home/jovyan/work/notebook_run/" +
                os.path.basename(notebook).replace(".ipynb", "") + "_" +
                self.timestamp_now() +
                # TODO params to string
                # TODO self.gconf.user()
                ".ipynb")

    def timestamp_now(self):
        """Depending on how the timezone is configured, returns the
         timestamp for now
        """
        d = datetime.utcnow().replace(tzinfo=timezone("UTC"))
        # TODO use config from self.gconf.get_timezone()
        pst = timezone('Europe/Paris')
        return d.astimezone(pst).strftime("%Y%m%d%H%M%S%f")

    def step_field(self, field):
        """Returns the value associated with the field for the focused step"""
        if self.has_step_field(field):
            return self.step[field]
        else:
            return None

    def has_step_field(self, field):
        """Tests if the focus step has a configuration value for the field"""
        if self.step is None:
            return False
        else:
            return field in self.step

    def file_path(self, string):
        """Returns absolute path of a file from a field of the focused step"""
        # TODO if string is url/git repo, download file locally first
        return os.path.abspath(self.rootDir + self.step_field(string))

    def container_name(self):
        """Return the docker container name to execute this step"""
        return (
            self.step_field("type") +
            "_" +
            self.step_field("name")
        )

    def extract_parameters(self):
        """Returns a list of docker parameters"""
        result = []
        if self.has_step_field("parameters"):
            for p in self.step_field("parameters"):
                for k in p:
                    result += ["-p", k, p[k]]
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Apache Software License 2.0
#
# Copyright (c) 2018, Christophe Duong
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The setup script."""
import os
from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

ROOT_DIR = os.getenv('VIRTUAL_ENV', '')
if len(ROOT_DIR) == 0:
    ROOT_DIR = os.getenv('HOME') + '/.aiscalator/'
else:
    ROOT_DIR += '/etc/aiscalator/'
data_files = [
    (ROOT_DIR + 'config',
     ['resources/config/logging.yaml']),
    (ROOT_DIR + 'config/docker/jupyter-spark/',
     ['resources/config/docker/jupyter-spark/Dockerfile']),
    (ROOT_DIR + 'config/docker/jupyter-spark/',
     ['resources/config/docker/jupyter-spark/install-jupytext.sh']),
    (ROOT_DIR + 'config/docker/airflow/',
     ['resources/config/docker/airflow/Dockerfile']),
    (ROOT_DIR + 'config/docker/airflow/config',
     ['resources/config/docker/airflow/config/airflow.cfg']),
    (ROOT_DIR + 'config/docker/airflow/config',
     ['resources/config/docker/airflow/config/docker-compose-CeleryExecutor.yml']),
    (ROOT_DIR + 'config/docker/airflow/config',
     ['resources/config/docker/airflow/config/docker-compose-LocalExecutor.yml']),
]

requirements = [
    'Click>=6.0',
    'pytz>=2018.5',
    'PyYAML>=3.13'
]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="Christophe Duong",
    author_email='chris@aiscalate.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Aiscalate Command Line Tool",
    entry_points={
        'console_scripts': [
            'aiscalator=aiscalator.cli:main',
        ],
    },
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='data science jupyter notebook prototype data engineering product airflow',
    name='aiscalator',
    packages=find_packages(),
    data_files=data_files,
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/Aiscalate/aiscalator',
    version='0.0.3',
    zip_safe=False,
)
